# Copyright (C) 2004-2006 struktur AG and Contributors <simon@struktur.de>
# http://www.strukturag.com, http://www.icoya.com

# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL). A copy of the ZPL should accompany this distribution.

# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.

# See LICENSE.txt for details

""" Class: SecureSessionAuthHelper

$Id: SessionAuthHelper.py 40169 2005-11-16 20:09:11Z tseaver $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import default__class_init__ as InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.PluggableAuthService.interfaces.plugins import \
        ILoginPasswordHostExtractionPlugin
from Products.PluggableAuthService.interfaces.plugins import \
        ICredentialsUpdatePlugin
from Products.PluggableAuthService.interfaces.plugins import \
        ICredentialsResetPlugin
        
from Products.PluggableAuthService.plugins.SessionAuthHelper import \
        SessionAuthHelper as BasePlugin, ISessionAuthHelper

from Products.PluggableAuthService.utils import classImplements
try:
    from Products.PluggableAuthService.utils import Interface
except ImportError:
    from zope.interface import Interface

from SessionSecureContainer import SessionSecureContainer
from sessionlazycheck import sessionlazycheck


# The session key where to store the secure container.
KEY = '__sa' 


class ISecureSessionAuthHelper(Interface):
    """ Marker interface.
    """


manage_addSecureSessionAuthHelperForm = PageTemplateFile(
    'dtml/addSSA', globals(), __name__='manage_addSecureSessionAuthHelperForm')


def manage_addSecureSessionAuthHelper(dispatcher, id, title=None, REQUEST=None):
    """ Add a Secure Session Auth Helper to a Pluggable Auth Service. """
    sp = SecureSessionAuthHelper(id, title)
    dispatcher._setObject(sp.getId(), sp)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( '%s/manage_workspace'
                                      '?manage_tabs_message='
                                      'SecureSessionAuthHelper+added.'
                                    % dispatcher.absolute_url() )


class SecureSessionAuthHelper(BasePlugin):
    """ Multi-plugin for managing details of Session Authentication. """
    meta_type = 'Secure Session Auth Helper'
    security = ClassSecurityInfo()

    security.declarePrivate('extractCredentials')
    def extractCredentials(self, request):
        """ Extract basic auth credentials from 'request'. """
       
        creds = {}
    
        # New login?
        name = request.get('__ac_name', None)
        if name:
        
            # Look in the request for the credentials coming from the
            # login form.
            try:
                name = request.get('__ac_name')
                password = request.get('__ac_password')
            except KeyError:
                name = password = None
                
            if name:
                # New login -> use these credentials.
                creds['login'] = name
                creds['password'] = password
    
        # Existing session credentials?
        if not creds and self.requestHasSession(request):
        
            session = request.SESSION
            
            try: 
                sec = session[KEY]
            except: 
                sec = None
                
            if isinstance(sec, SessionSecureContainer):
            
                try:
                    name = sec.get('name')
                    password = sec.get('password')
                except KeyError:
                    name = password = None
                    
                if name:
                    creds['login'] = name
                    creds['password'] = password

        """
        # No auth until now.
        if not creds:
        
            # Look into the request now
            login_pw = request._authUserPW()
            
            if login_pw is not None:
                name, password = login_pw
                creds['login'] = name
                creds['password'] = password
                
                # Store basic auth credentials as session auth.
                self.updateCredentials(request, request.RESPONSE, \
                        name, password)
        """

        # Finally try to get more data when there are credentials.
        if creds:
     
            creds['remote_host'] = request.get('REMOTE_HOST', '')
            
            try:
                creds['remote_address'] = request.getClientAddr()
            except AttributeError:
                creds['remote_address'] = request.get('REMOTE_ADDR', '')

        return creds        
    
    security.declarePrivate('updateCredentials')
    def updateCredentials(self, request, response, login, new_password):
        """ Respond to change of credentials. """

        session = request.SESSION
        
        # Get or create secure session container.
        try: 
            sec = session[KEY]
        except: 
            sec = SessionSecureContainer()
        else:
            if not isinstance(sec, SessionSecureContainer):
                sec = SessionSecureContainer()
        
        # Store credentials to secure container.
        sec.set('name', login)
        sec.set('password', new_password)        
        
        # Store container into session.
        session[KEY]=sec

    security.declarePrivate('resetCredentials')
    def resetCredentials(self, request, response):
        """ Empty out the currently-stored session values """

        if self.requestHasSession(request):
            
            session = request.SESSION
        
            # Only reset when already some auth is stored.
            try:
                session[KEY]
            except:
                pass
            else:
                session[KEY] = None

    security.declarePublic('requestHasSession')
    def requestHasSession(self, req):
        # Check for a session without creating a new one.
        # (returns True or False)
        return sessionlazycheck(self)
        

classImplements( SecureSessionAuthHelper
               , ISecureSessionAuthHelper
               , ISessionAuthHelper
               , ILoginPasswordHostExtractionPlugin
               , ICredentialsUpdatePlugin
               , ICredentialsResetPlugin
               )


InitializeClass(SecureSessionAuthHelper)
